/*******************************************************************
 * 							 Pilot Software
 *
 *		 Copyright(c) 1995-1998, Palm Computing Inc., All Rights Reserved
 *
 *-------------------------------------------------------------------  
 * FileName:
 *		M68328Hwr.h
 *
 * Description:
 *		Hardware Equates for the Motorola 68EZ328 Dragonball-EZ chip 
 *
 * History:
 *   	11/13/97  srj- Created from M68328Hwr.h
 *
 *******************************************************************/  

#ifdef	NON_PORTABLE						// So apps don't include this...


#ifndef	__M68EZ328HWR_H__
#define	__M68EZ328HWR_H__


#include <Common.h>							// Touchdown includes


/***********************************************************************
 * Motorola 68328  registers
 ***********************************************************************/
typedef struct HwrDBallType {

	Byte		scr;								// $000: System Control Register
	Byte											___filler0[0x004-0x001];
	Byte		chipID;							// $004: Chip ID Register
	Byte		maskID;							// $005: Mask ID Register
	Word		swID;								// $006: Software ID Register
	Byte											___filler1[0x100-0x008];				 
	
	Word		csAGroupBase;					// $100: Chip Select Group A Base Register
	Word		csBGroupBase;					// $102: Chip Select Group B Base Register
	Word		csCGroupBase;					// $104: Chip Select Group C Base Register
	Word		csDGroupBase;					// $106: Chip Select Group D Base Register

	Byte											___filler6[0x110-0x108];

	Word		csASelect;						// $110: Group A Chip Select Register
	Word		csBSelect;						// $112: Group B Chip Select Register
	Word		csCSelect;						// $114: Group C Chip Select Register
	Word		csDSelect;						// $116: Group D Chip Select Register
	
	Word		emuCS;						   // $118: EMU Chip Select Register
	
	Byte											___filler2[0x200-0x11A];		
	
	Word		pllControl;						// $200: PLL Control Register
	Word		pllFreqSel;						// $202: PLL Frequency Select Register
	Word		pllTest;							// $204: PLL Test Register (do not access)
	Byte											___filler44;
	Byte		pwrControl;						// $207: Power Control Register
	
	Byte											___filler3[0x300-0x208];
	
	Byte		intVector;						// $300: Interrupt Vector Register
	Byte											___filler4;
	Word		intControl;						// $302: Interrupt Control Register
	Word		intMaskHi;						// $304: Interrupt Mask Register/HIGH word
	Word		intMaskLo;						// $306: Interrupt Mask Register/LOW word
	Byte											___filler7[0x30c-0x308];
	Word		intStatusHi;					// $30C: Interrupt Status Register/HIGH word
	Word		intStatusLo;					// $30E: Interrupt Status Register/LOW word
	Word		intPendingHi;					// $310: Interrupt Pending Register
	Word		intPendingLo;					// $312: Interrupt Pending Register
	
	Byte 											___filler4a[0x400-0x314];
	
	Byte		portADir;						// $400: Port A Direction Register
	Byte		portAData;						// $401: Port A Data Register
	Byte		portAPullupEn;					// $402: Port A Pullup Enable (similar to Select on DB)
	Byte											___filler8[5];	
	
	Byte		portBDir;						// $408: Port B Direction Register
	Byte		portBData;						// $409: Port B Data Register
	Byte		portBPullupEn;					// $40A: Port B Pullup Enable
	Byte		portBSelect;					// $40B: Port B Select Register	
	
	Byte											___filler9[4];
	
	Byte		portCDir;						// $410: Port C Direction Register
	Byte		portCData;						// $411: Port C Data Register
	Byte		portCPulldnEn;					// $412: Port C Pulldown Enable
	Byte		portCSelect;					// $413: Port C Select Register	
	
	Byte											___filler10[4];
	
	Byte		portDDir;						// $418: Port D Direction Register
	Byte		portDData;						// $419: Port D Data Register
	Byte		portDPullupEn;					// $41A: Port D Pull-up Enable
	Byte		portDSelect;					// $41B: Port D Select Register
	Byte		portDPolarity;					// $41C: Port D Polarity Register
	Byte		portDIntReqEn;					// $41D: Port D Interrupt Request Enable
	Byte		portDKbdIntEn;					// $41E: Port D Keyboard Interrupt Enable
	Byte		portDIntEdge;					// $41F: Port D IRQ Edge Register
	
	Byte		portEDir;						// $420: Port E Direction Register
	Byte		portEData;						// $421: Port E Data Register
	Byte		portEPullupEn;					// $422: Port E Pull-up Enable
	Byte		portESelect;					// $423: Port E Select Register
	
	Byte											___filler14[4];
	
	Byte		portFDir;						// $428: Port F Direction Register
	Byte		portFData;						// $429: Port F Data Register
	Byte		portFPullupdnEn;				// $42A: Port F Pull-up/down Enable
	Byte		portFSelect;					// $42B: Port F Select Register
	
	Byte											___filler16[4];
		
	Byte		portGDir;						// $430: Port G Direction Register
	Byte		portGData;						// $431: Port G Data Register
	Byte		portGPullupEn;					// $432: Port G Pull-up Enable
	Byte		portGSelect;					// $433: Port G Select Register
	
	Byte											___filler2000[0x500-0x434];

	Word		pwmControl;						// $500: PWM Control Register
	Byte		pwmSampleHi;    				// $502: PWM Sample - high byte
	Byte		pwmSampleLo;					// $503: PWM Sample - low byte
	Byte		pwmPeriod;						// $504: PWM Period
	Byte		pwmCounter;						// $505: PWM Counter
	
	Byte											___filler24[0x600-0x506];
	
	Word		tmr1Control;					// $600: Timer 1 Control Register
	Word		tmr1Prescaler;					// $602: Timer 1 Prescaler Register
	Word		tmr1Compare;					// $604: Timer 1 Compare Register
	Word		tmr1Capture;					// $606: Timer 1 Capture Register
	Word		tmr1Counter;					// $608: Timer 1 Counter Register
	Word		tmr1Status;						// $60A: Timer 1 Status Register
	
	Byte											___filler25[0x800-0x60C];
		
	Word		spiMasterData;					// $800: SPI Master Data Register
	Word		spiMasterControl;				// $802: SPI Master Control Register
	
	Byte											___filler27[0x900-0x804];
	
	Word		uControl;						// $900: Uart Control Register
	Word		uBaud;							// $902: Uart Baud Control Register
	Word		uReceive;						// $904: Uart Receive Register
	Word		uTransmit;						// $906: Uart Transmit Register
	Word		uMisc;							// $908: Uart Miscellaneous Register
	Word		uNonIntPresc;					// $90A: Uart IRDA Non-Integer Prescaler
	
	Byte											___filler28[0xA00-0x90C];
	
	DWord		lcdStartAddr;					// $A00: Screen Starting Address Register
	Byte											___filler29;
	Byte		lcdPageWidth;					// $A05: Virtual Page Width Register
	Byte											___filler30[2];
	Word		lcdScreenWidth;				// $A08: Screen Width Register
	Word		lcdScreenHeight;				// $A0A: Screen Height Register
	Byte											___filler31[0xA18-0xA0C];
	Word		lcdCursorXPos;					// $A18: Cursor X Position
	Word		lcdCursorYPos;					// $A1A:	Cursor Y Position
	Word		lcdCursorWidthHeight;		// $A1C: Cursor Width and Height
	Byte											___filler32;
	Byte		lcdBlinkControl;				// $A1F: Blink Control Register
	Byte		lcdPanelControl;				// $A20: Panel Interface Control Register
	Byte		lcdPolarity;					// $A21: Polarity Config Register
	Byte											___filler33;						
	Byte		lcdACDRate;						// $A23: ACD (M) Rate Control Register
	Byte											___filler34;
	Byte		lcdPixelClock;					// $A25: Pixel Clock Divider Register
	Byte											___filler35;
	Byte		lcdClockControl;				// $A27: Clocking Control Register
	Byte											___filler36;
	Byte		lcdRefreshRateAdj;			// $A29: Refresh Rate Adjustment Register
	Byte											___filler2003[0xA2D-0xA2A];
	Byte     lcdPanningOffset;				// $A2D: Panning Offset Register
	
	Byte											___filler37[0xA31-0xA2E];
	
	Byte		lcdFrameRate;					// $A31: Frame Rate Control Modulation Register
	Byte											___filler2004;
	Byte		lcdGrayPalette;				// $A33: Gray Palette Mapping Register
	Byte		lcdReserved;					// $A34: Reserved
	Byte											___filler2005;
	Word		lcdContrastControlPWM;		// $A36: Contrast Control
	
	Byte											___filler40[0xB00-0xA38];
	
	DWord		rtcHourMinSec;					// $B00: RTC Hours, Minutes, Seconds Register
	DWord		rtcAlarm;						// $B04: RTC Alarm Register
	Byte											___filler2001[0xB0A-0xB08];
	Word     rtcWatchDog;               // $B0A: RTC Watchdog Timer
	Word		rtcControl;						// $B0C: RTC Control Register
	Word		rtcIntStatus;					// $B0E: RTC Interrupt Status Register
	Word		rtcIntEnable;					// $B10: RTC Interrupt Enable Register
	Word		stopWatch;						// $B12: Stopwatch Minutes
	Byte											___filler2002[0xB1A-0xB14];
	Word		rtcDay;							// $B1A: RTC Day
	Word     rtcDayAlarm;					// $B1C: RTC Day Alarm
	
   Byte											___filler41[0xC00-0xB1E];

   Word		dramConfig;                // $C00: DRAM Memory Config Register
   Word     dramControl;               // $C02: DRAM Control Register

   Byte                                ___filler42[0xD00-0xC04];
	
	DWord		emuAddrCompare;				// $D00: Emulation Address Compare Register   
	DWord		emuAddrMask;					// $D04: Emulation Address Mask Register
	Word		emuControlCompare;			// $D08: Emulation Control Compare Register
	Word		emuControlMask;				// $D0A: Emulation Control Mask Register
	Word		emuControl;						// $DOC: Emulation Control Register
	Word		emuStatus;						// $D0E: Emulation Status Register
	   
  	} HwrDBallType;

typedef	volatile   HwrDBallType*	HwrDBallPtr;

/************************************************************************
 * Chip ID register for EZ
 ***********************************************************************/
 #define hwrEZ328chipIDEZ		0x45

/************************************************************************
 * Mask ID register for EZ
 ***********************************************************************/
 #define hwrEZ328maskID0H31J	0x01
 #define hwrEZ328maskID0J75C	0x02
 #define hwrEZ328maskID1J75C	0x03
 #define hwrEZ328maskID0J83G	0x04
 
/************************************************************************
 * LCD Controller Bits
 ************************************************************************/
// lcdCursorXPos Register
#define	hwrEZ328LcdCursorXPosCtlMask			0xC000
	#define	hwrEZ328LcdCursorXPosCtlTrans	0x0000
	#define	hwrEZ328LcdCursorXPosCtlBlack	0x4000
	#define	hwrEZ328LcdCursorXPosCtlReverse	0x8000
	#define  hwrEZ328LcdCursorXPosCtlWhite   0xC000
#define  hwrEZ328LcdCursorXPosValueMask		0x1FF
#define  hwrEZ328LcdCursorYPosValueMask		0x1FF
	
#define hwrEZ328LcdCursorWidthMask			0x1F
   #define  hwrEZ328LcdCursorWidthOffset  0x8
#define hwrEZ328LcdCursorHeightMask			0x1F

// lcdBlinkControl Register
#define	hwrEZ328LcdBlinkControlEnable		0x80 
#define  hwrEZ328LcdBlinkDivisorMask      0x7F

// lcdPanelControl Register
#define	hwrEZ328LcdPanelControlBusMask		0x0C
	#define	hwrEZ328LcdPanelControlBus1Bit	0x00
	#define	hwrEZ328LcdPanelControlBus2Bit	0x04
	#define	hwrEZ328LcdPanelControlBus4Bit	0x08
#define	hwrEZ328LcdPanelControlGrayScale	0x01

// lcdPolarity Register
#define	hwrEZ328LcdPolarityShiftClock		0x08
#define	hwrEZ328LcdPolarityFLM					0x04
#define	hwrEZ328LcdPolarityLP					0x02
#define	hwrEZ328LcdPolarityPixel				0x01

// lcdClockControl Register
#define	hwrEZ328LcdClockControlEnable			0x80
#define	hwrEZ328LcdClockControl8BitBus			0x40

#define	hwrEZ328LcdClockControlBurstRateMask	0x0F
#define	hwrEZ328LcdClockControlBurstRate1		0x00
#define	hwrEZ328LcdClockControlBurstRate2		0x01
#define	hwrEZ328LcdClockControlBurstRate3		0x02
#define	hwrEZ328LcdClockControlBurstRate4		0x03
#define	hwrEZ328LcdClockControlBurstRate5		0x04
#define	hwrEZ328LcdClockControlBurstRate6		0x05
#define	hwrEZ328LcdClockControlBurstRate7		0x06
#define	hwrEZ328LcdClockControlBurstRate8		0x07
#define	hwrEZ328LcdClockControlBurstRate9		0x08
#define	hwrEZ328LcdClockControlBurstRate10		0x09
#define	hwrEZ328LcdClockControlBurstRate11		0x0A
#define	hwrEZ328LcdClockControlBurstRate12		0x0B
#define	hwrEZ328LcdClockControlBurstRate13		0x0C
#define	hwrEZ328LcdClockControlBurstRate14		0x0D
#define	hwrEZ328LcdClockControlBurstRate15		0x0E
#define	hwrEZ328LcdClockControlBurstRate16		0x0F

#define  hwrEZ328LcdContrastPWMWidthMask			0x00FF
#define  hwrEZ328LcdContrastEnable					0x0100



/************************************************************************
 * Interrupt Controller Bits
 ************************************************************************/
// intMaskHi bits
// NOTE: Due to a bug in early rev's of the DragonBall, the clear bits for
// IRQ[1236] on those early chips were different than the indicator bits
// (the order was reversed).  So some older code uses the OR of the IRQ1 and
// IRQ6 bits for either, and the OR of the IRQ2 and IRQ3 bits for either.
#define	hwrEZ328IntHiEMU							0x0080
#define	hwrEZ328IntHiSampleTimer				0x0040
#define	hwrEZ328IntHiPen							0x0010
#define	hwrEZ328IntHiIRQ6						0x0008
#define	hwrEZ328IntHiIRQ3						0x0004
#define	hwrEZ328IntHiIRQ2						0x0002
#define	hwrEZ328IntHiIRQ1						0x0001

// intMaskLo bits
#define	hwrEZ328IntLoInt3						0x0800
#define	hwrEZ328IntLoInt2						0x0400
#define	hwrEZ328IntLoInt1						0x0200
#define	hwrEZ328IntLoInt0						0x0100
#define	hwrEZ328IntLoAllKeys					0x0F00
#define	hwrEZ328IntLoInt0Bit					8
#define	hwrEZ328IntLoPWM						0x0080
#define	hwrEZ328IntLoPWMBit					7
#define	hwrEZ328IntLoKbd						0x0040
#define	hwrEZ328IntLoRTC						0x0010
#define	hwrEZ328IntLoRTCBit					4
#define	hwrEZ328IntLoWDT						0x0008
#define	hwrEZ328IntLoUART						0x0004
#define	hwrEZ328IntLoUARTBit					2
#define	hwrEZ328IntLoTimer					0x0002
#define	hwrEZ328IntLoTimerBit				1
#define	hwrEZ328IntLoSPIM						0x0001

// intControl bits
// NOTE: These are different than that documented in the User's manual 
// for the DragonBall (see erratas).  The DragonBall is the same as the EZ.
#define	hwrEZ328IntCtlEdge1						0x0800	
#define	hwrEZ328IntCtlEdge2						0x0400
#define	hwrEZ328IntCtlEdge3						0x0200
#define	hwrEZ328IntCtlEdge6						0x0100
#define	hwrEZ328IntCtlPol1						0x8000	
#define	hwrEZ328IntCtlPol2						0x4000
#define	hwrEZ328IntCtlPol3						0x2000
#define	hwrEZ328IntCtlPol6						0x1000
#define  hwrEZ328IntCtlPol5						0x0080

/************************************************************************
 * DRAM Controller Bits bits
 * Not all the bits are defined here-- some are pretty bad to muck with
 * in anything but initial setup of the DRAM.
 ************************************************************************/
#define  hwrEZ328dramControlEnable					0x8000
#define  hwrEZ328dramControlSelfRefreshEn			0x4000
#define  hwrEZ328dramControlLightSleep				0x0010
#define  hwrEZ328dramControlLowPowerRefreshEn		0x0004
#define  hwrEZ328dramControlDWEEnable				0x0001


/************************************************************************
 * Timer bits
 ************************************************************************/
#define	hwrEZ328TmrControlUnused						0xFC00
#define  hwrEZ328TmrControlTIOOutput              0x0200
#define	hwrEZ328TmrControlFreeRun						0x0100
#define	hwrEZ328TmrControlCaptureEdgeMask			0x00C0
	#define	hwrEZ328TmrControlCaptureEdgeNone		0x0000
	#define	hwrEZ328TmrControlCaptureEdgeRising	0x0040
	#define	hwrEZ328TmrControlCaptureEdgeFalling	0x0080
	#define	hwrEZ328TmrControlCaptureEdgeBoth		0x00C0
#define	hwrEZ328TmrControlOutputModeToggle			0x0020
#define	hwrEZ328TmrControlEnInterrupt				0x0010
#define	hwrEZ328TmrControlClkSrcMask					0x000E
	#define	hwrEZ328TmrControlClkSrcStop				0x0000
	#define	hwrEZ328TmrControlClkSrcSys				0x0002
	#define	hwrEZ328TmrControlClkSrcSysBy16			0x0004
	#define	hwrEZ328TmrControlClkSrcTIN				0x0006
	#define	hwrEZ328TmrControlClkSrc32KHz			0x0008
#define	hwrEZ328TmrControlEnable						0x0001

#define	hwrEZ328TmrStatusCapture						0x0002
#define	hwrEZ328TmrStatusCaptureBit					1
#define	hwrEZ328TmrStatusCompare						0x0001
#define	hwrEZ328TmrStatusCompareBit					0


/************************************************************************
 * 68EZ328 Serial Port Register Bits
 ************************************************************************/
// uControl Register
#define	hwrEZ328UControlUARTEnable			0x8000
#define	hwrEZ328UControlRxEnable				0x4000
#define	hwrEZ328UControlTxEnable				0x2000
#define	hwrEZ328UControlRxClock1xSync		0x1000
#define	hwrEZ328UControlParityEn				0x0800
#define	hwrEZ328UControlParityOdd				0x0400
#define	hwrEZ328UControlStopBits2				0x0200
#define	hwrEZ328UControlDataBits8				0x0100
#define	hwrEZ328UControlOldDataEn				0x0080
#define	hwrEZ328UControlCTSDeltaEn			0x0040
#define	hwrEZ328UControlRxFullEn				0x0020
#define	hwrEZ328UControlRxHalfEn				0x0010
#define	hwrEZ328UControlRxRdyEn				0x0008
#define	hwrEZ328UControlTxEmptyEn				0x0004
#define	hwrEZ328UControlTxHalfEn				0x0002
#define	hwrEZ328UControlTxAvailEn				0x0001

#define	hwrEZ328UControlEnableAll			   (hwrEZ328UControlUARTEnable |  	\
														 hwrEZ328UControlRxEnable |		\
														 hwrEZ328UControlTxEnable)

// uBaud Register
// UCLK output requires PE3 Select to be 0
#define	hwrEZ328UBaudUCLKDirOut				0x2000
#define	hwrEZ328UBaudBaudSrcUCLK				0x0800
#define	hwrEZ328UBaudDivider					0x0700
#define	hwrEZ328UBaudPrescaler					0x003F

#define	hwrEZ328UBaudDivideBitOffset			8


// uReceive Register
#define	hwrEZ328UReceiveFIFOFull				0x8000
#define	hwrEZ328UReceiveFIFOHalf				0x4000
#define	hwrEZ328UReceiveDataRdy				0x2000
#define	hwrEZ328UReceiveDataRdyBit			13
#define	hwrEZ328UReceiveOldData				0x1000
#define	hwrEZ328UReceiveOverrunErr			0x0800
#define	hwrEZ328UReceiveOverrunErrBit		11
#define	hwrEZ328UReceiveFrameErr				0x0400
#define	hwrEZ328UReceiveFrameErrBit			10
#define	hwrEZ328UReceiveBreakErr				0x0200
#define	hwrEZ328UReceiveBreakErrBit			9
#define	hwrEZ328UReceiveParityErr				0x0100
#define	hwrEZ328UReceiveParityErrBit			8
#define	hwrEZ328UReceiveData					0x00FF

#define	hwrEZ328UReceiveErrsMask				(hwrEZ328UReceiveOverrunErr |	\
														 hwrEZ328UReceiveFrameErr |		\
														 hwrEZ328UReceiveBreakErr |		\
														 hwrEZ328UReceiveParityErr)

// uTransmit Register
#define	hwrEZ328UTransmitFIFOEmpty			0x8000
#define	hwrEZ328UTransmitFIFOHalf				0x4000
#define	hwrEZ328UTransmitTxAvail				0x2000
#define	hwrEZ328UTransmitSendBreak			0x1000
#define	hwrEZ328UTransmitIgnoreCTS			0x0800
#define  hwrEZ328UTransmitBusy              0x0400
#define	hwrEZ328UTransmitCTSStatus			0x0200
#define	hwrEZ328UTransmitCTSDelta				0x0100
#define	hwrEZ328UTransmitData					0x00FF

// uMisc Register
#define  hwrEZ328UMiscBaudTest              0x8000
#define	hwrEZ328UMiscClkSrcUCLK				0x4000
#define	hwrEZ328UMiscForceParityErr			0x2000
#define	hwrEZ328UMiscLoopback					0x1000
#define  hwrEZ328UMiscBaudReset             0x0800
#define  hwrEZ328UMiscIRTestEn              0x0400
#define	hwrEZ328UMiscReservedMask				0x0300
#define	hwrEZ328UMiscRTSThruFIFO				0x0080
#define	hwrEZ328UMiscRTSOut						0x0040
#define	hwrEZ328UMiscIRDAEn						0x0020
#define	hwrEZ328UMiscLoopIRDA					0x0010
#define  hwrEZ328UMiscRXPolarityInv         0x0008
#define  hwrEZ328UMiscTXPolarityInv         0x0004
#define	hwrEZ328UMiscUnused						0x0003

// uNonIntPresc Register
#define	hwrEZ328UNonIntPrescSrcIntPresc		0x0000
#define	hwrEZ328UNonIntPrescSrcNonIntPresc	0x8000

#define	hwrEZ328UNonIntPrescSelDiv128			0x0000
#define	hwrEZ328UNonIntPrescSelDiv64			0x0100
#define	hwrEZ328UNonIntPrescSelDiv32			0x0200
#define	hwrEZ328UNonIntPrescSelDiv16			0x0300
#define	hwrEZ328UNonIntPrescSelDiv8			0x0400
#define	hwrEZ328UNonIntPrescSelDiv4			0x0500
#define	hwrEZ328UNonIntPrescSelDiv2			0x0600
#define	hwrEZ328UNonIntPrescSelDiv1			0x0700

#define	hwrEZ328UNonIntPrescStepMask			0x00FF

// see EZ docs for explanation of the following.  This provides the correct
// pulse width for IRDA, and is just a statement in the docs. The values don't
// appear to make sense, but I just push the buttons.

#define 	hwrEZ328UNonIntPrescIRDAMode			(hwrEZ328UNonIntPrescSrcIntPresc \
															| hwrEZ328UNonIntPrescSelDiv32 \
															| (hwrEZ328UNonIntPrescStepMask & 0x20))


/************************************************************************
 * PWM Bits
 ************************************************************************/
// PWM register bits of interest
#define	hwrEZ328PWMControlEnable				0x0010
#define	hwrEZ328PWMControlEnableBit			4
#define	hwrEZ328PWMControlEnableIRQ			0x0040
#define	hwrEZ328PWMIRQStatus					0x0080
// PWM divider chain output selectors
#define	hwrEZ328PWMControlDivMask				0x0003
#define	hwrEZ328PWMControlDivBy2				0x0
#define	hwrEZ328PWMControlDivBy4				0x1
#define	hwrEZ328PWMControlDivBy8				0x2
#define	hwrEZ328PWMControlDivBy16				0x3
// PWM Repeat Counts
#define	hwrEZ328PWMRepeatSample1x				0x0
#define	hwrEZ328PWMRepeatSample2x				0x4
#define	hwrEZ328PWMRepeatSample4x				0x8
#define	hwrEZ328PWMRepeatSample8x				0xC
// PWM hardware constraints
#define	hwrEZ328PWMFifoSizeInBytes				4
#define	hwrEZ328PWMPrescalerMask				0x7F00
#define	hwrEZ328PWMPrescalerOffset				8

/************************************************************************
 * PLL Bits
 * note that these do not follow the '328 spec-- however, they seem to 
 * follow the 'EZ328 spec, so maybe Motorola is behind in their tech pubs
 * department?? (Naah.)
 ************************************************************************/
#define	hwrEZ328PLLControlDisable				0x0008
#define	hwrEZ328PLLControlClkEn				0x0010

#define	hwrEZ328PLLControlSysDMADiv2			0x0000
#define	hwrEZ328PLLControlSysDMADiv4			0x0100
#define	hwrEZ328PLLControlSysDMADiv8			0x0200
#define	hwrEZ328PLLControlSysDMADiv16		0x0300
#define	hwrEZ328PLLControlSysDMADiv1			0x0400

#define	hwrEZ328PLLControlPixDMADiv2			0x0000  
#define	hwrEZ328PLLControlPixDMADiv4			0x0800
#define	hwrEZ328PLLControlPixDMADiv8			0x1000
#define	hwrEZ328PLLControlPixDMADiv16		0x1800
#define	hwrEZ328PLLControlPixDMADiv1			0x2000

#define  hwrEZ328PLLControlDMAVCODiv2       0x0000
#define  hwrEZ328PLLControlDMAVCODiv4       0x0020
#define  hwrEZ328PLLControlDMAVCODiv8       0x0040
#define  hwrEZ328PLLControlDMAVCODiv16      0x0060
#define  hwrEZ328PLLControlDMAVCODiv1       0x0080

#define  hwrEZ328PLLFreqSelCLK32				0x8000

/************************************************************************
 * Real Time Clock (RTC) Bits
 ************************************************************************/

// RTC Control Register (RTCCTL) bits
#define hwrEZ328RTCControlRTCEnable				0x80
#define hwrEZ328RTCControlRefSelMask				0x20
#define hwrEZ328RTCControlRefSel38400				0x20		// 38.4 kHz reference frequency
#define hwrEZ328RTCControlRefSel32768				0x00		// 32.768 kHz reference frequency

// RTC Interrupt Enable Register (RTCIENR) bits
#define hwrEZ328RTCIntEnableSample7            0x8000
#define hwrEZ328RTCIntEnableSample6            0x4000
#define hwrEZ328RTCIntEnableSample5            0x2000
#define hwrEZ328RTCIntEnableSample4            0x1000
#define hwrEZ328RTCIntEnableSample3            0x0800
#define hwrEZ328RTCIntEnableSample2            0x0400
#define hwrEZ328RTCIntEnableSample1            0x0200
#define hwrEZ328RTCIntEnableSample0            0x0100
#define hwrEZ328RTCIntEnableHour               0x0020
#define hwrEZ328RTCIntEnableSec						0x0010
#define hwrEZ328RTCIntEnable24Hr					0x0008
#define hwrEZ328RTCIntEnableAlarm					0x0004
#define hwrEZ328RTCIntEnableMinute					0x0002
#define hwrEZ328RTCIntEnableStopWatch				0x0001

// RTC Interrupt Status Register bits
#define hwrEZ328RTCIntStatusSample7            0x8000
#define hwrEZ328RTCIntStatusSample6            0x4000
#define hwrEZ328RTCIntStatusSample5            0x2000
#define hwrEZ328RTCIntStatusSample4            0x1000
#define hwrEZ328RTCIntStatusSample3            0x0800
#define hwrEZ328RTCIntStatusSample2            0x0400
#define hwrEZ328RTCIntStatusSample1            0x0200
#define hwrEZ328RTCIntStatusSample0            0x0100
#define hwrEZ328RTCIntStatusHour               0x0020
#define hwrEZ328RTCIntStatusSec						0x0010
#define hwrEZ328RTCIntStatus24Hr					0x0008
#define hwrEZ328RTCIntStatusAlarm					0x0004
#define hwrEZ328RTCIntStatusMinute					0x0002
#define hwrEZ328RTCIntStatusStopWatch				0x0001

// RTC Alarm register (RTCALRM) masks and offsets
#define hwrEZ328RTCAlarmSecondsMask				0x0000003fL
#define hwrEZ328RTCAlarmSecondsOffset				0
#define hwrEZ328RTCAlarmMinutesMask				0x003f0000L
#define hwrEZ328RTCAlarmMinutesOffset				16
#define hwrEZ328RTCAlarmHoursMask					0x1f000000L
#define hwrEZ328RTCAlarmHoursOffset				24

// RTC hours/minutes/seconds register (RTCHMS) masks and offsets
#define hwrEZ328RTCHourMinSecSecondsMask			0x0000003fL
#define hwrEZ328RTCHourMinSecSecondsOffset		0
#define hwrEZ328RTCHourMinSecMinutesMask			0x003f0000L
#define hwrEZ328RTCHourMinSecMinutesOffset		16
#define hwrEZ328RTCHourMinSecHoursMask			0x1f000000L
#define hwrEZ328RTCHourMinSecHoursOffset			24

// RTC Day register masks/offsets
#define hwrEZ328RTCDayDayMask                  0x01FF
#define hwrEZ328RTCDayDayOffset                0

// RTC Watchdog Timer masks/offsets
#define hwrEZ328RTCWatchdogEnable              0x0001
#define hwrEZ328RTCWatchdogIntSel              0x0002
#define hwrEZ328RTCWatchdogIntStatus           0x0080
#define hwrEZ328RTCWatchdogCountMask           0x0300
#define hwrEZ328RTCWatchdogCountOffset         8

/************************************************************************
 * SPI Master bits
 ************************************************************************/
#define	hwrEZ328SPIMControlRateMask			0xE000
	#define	hwrEZ328SPIMControlRateDiv4			0x0000
	#define	hwrEZ328SPIMControlRateDiv8			0x2000
	#define	hwrEZ328SPIMControlRateDiv16			0x4000
	#define	hwrEZ328SPIMControlRateDiv32			0x6000
	#define	hwrEZ328SPIMControlRateDiv64			0x8000
	#define	hwrEZ328SPIMControlRateDiv128		0xA000
	#define	hwrEZ328SPIMControlRateDiv256		0xC000
	#define	hwrEZ328SPIMControlRateDiv512		0xE000
#define	hwrEZ328SPIMControlEnable				0x0200
#define	hwrEZ328SPIMControlExchange			0x0100
#define	hwrEZ328SPIMControlIntStatus			0x0080
#define	hwrEZ328SPIMControlIntEnable			0x0040
#define	hwrEZ328SPIMControlOppPhase			0x0020
#define	hwrEZ328SPIMControlInvPolarity		0x0010
#define	hwrEZ328SPIMControlBitsMask			0x000F


#endif // __M68EZ328_H__

#endif // NON_PORTABLE











